# -*- coding: utf-8 -*-
import datetime

DATABASE_URL = ("https://s3.eu-central-1.amazonaws.com/"
                "chanjo/ccds.20150512.sqlite3")
BED_URL = "https://s3.eu-central-1.amazonaws.com/chanjo/ccds.20150512.bed.zip"
CCDS_DATE = datetime.date(2015, 5, 12)

ARCHIVE = {}
